@@ -1,13 +1,16 @@
 from typing import List
 
-from sqlalchemy import func
+from sqlalchemy import func, null
 
 from conda_store_server import orm
 from .conda import conda_platform
 
 
-def list_namespaces(db):
+def list_namespaces(db, show_soft_deleted: bool = False):
     filters = []
+    if not show_soft_deleted:
+        filters.append(orm.Namespace.deleted_on == null())
+
     return db.query(orm.Namespace).filter(*filters)
 
 
@@ -20,11 +23,33 @@ def get_namespace(db, name: str = None, id: int = None):
     return db.query(orm.Namespace).filter(*filters).first()
 
 
-def list_environments(db, namespace: str = None, search=None):
+def create_namespace(db, name: str):
+    namespace = orm.Namespace(name=name)
+    db.add(namespace)
+
+
+def delete_namespace(db, name: str = None, id: int = None):
+    namespace = get_namespace(db, name=name, id=id)
+    if namespace:
+        db.delete(namespace)
+
+
+def list_environments(
+    db,
+    namespace: str = None,
+    search: str = None,
+    show_soft_deleted: bool = False,
+):
     filters = []
     if namespace:
         filters.append(orm.Namespace.name == namespace)
 
+    if search:
+        filters.append(orm.Environment.name.contains(search, autoescape=True))
+
+    if not show_soft_deleted:
+        filters.append(orm.Environment.deleted_on == null())
+
     return db.query(orm.Environment).join(orm.Environment.namespace).filter(*filters)
 
 
@@ -48,18 +73,9 @@ def get_environment(
     return db.query(orm.Environment).join(orm.Namespace).filter(*filters).first()
 
 
-def get_environment_builds(db, namespace, name):
-    return (
-        db.query(orm.Build)
-        .join(orm.Specification, orm.Build.specification_id == orm.Specification.id)
-        .join(orm.Namespace, orm.Build.namespace_id == orm.Namespace.id)
-        .filter(orm.Namespace.name == namespace, orm.Specification.name == name)
-        .all()
-    )
-
-
 def list_specifications(db, search=None):
-    return db.query(orm.Specification).all()
+    filters = []
+    return db.query(orm.Specification).filter(*filters)
 
 
 def get_specification(db, sha256):
@@ -72,25 +88,37 @@ def post_specification(conda_store, specification, namespace=None):
     conda_store.register_environment(specification, namespace)
 
 
-def list_builds(db, status: orm.BuildStatus = None):
+def list_builds(db, status: orm.BuildStatus = None, show_soft_deleted: bool = False):
     filters = []
     if status:
         filters.append(orm.Build.status == status)
 
+    if not show_soft_deleted:
+        filters.append(orm.Build.deleted_on == null())
+
     return db.query(orm.Build).filter(*filters)
 
 
-def get_build(db, build_id):
+def get_build(db, build_id: int):
     return db.query(orm.Build).filter(orm.Build.id == build_id).first()
 
 
-def get_num_queued_builds(db, status):
+def get_build_packages(db, build_id: int, search: str = None, build: str = None):
+    filters = [(orm.build_conda_package.c.build_id == build_id)]
+    if search:
+        filters.append(orm.CondaPackage.name.contains(search, autoescape=True))
+    if build:
+        filters.append(orm.CondaPackage.build.contains(build, autoescape=True))
+
     return (
-        db.query(orm.Build).filter(orm.Build.status == orm.BuildStatus.QUEUED).count()
+        db.query(orm.CondaPackage)
+        .join(orm.build_conda_package)
+        .join(orm.CondaChannel)
+        .filter(*filters)
     )
 
 
-def get_build_lockfile(db, build_id):
+def get_build_lockfile(db, build_id: int):
     build = db.query(orm.Build).filter(orm.Build.id == build_id).first()
     packages = [
         f"{row.channel.name}/{row.subdir}/{row.name}-{row.version}-{row.build}.tar.bz2#{row.md5}"
@@ -104,7 +132,7 @@ def get_build_lockfile(db, build_id):
     )
 
 
-def get_build_artifact_types(db, build_id):
+def get_build_artifact_types(db, build_id: int):
     return (
         db.query(orm.BuildArtifact.artifact_type)
         .filter(orm.BuildArtifact.build_id == build_id)
@@ -114,7 +142,6 @@ def get_build_artifact_types(db, build_id):
 
 def list_build_artifacts(
     db,
-    limit: int = 25,
     build_id: int = None,
     key: str = None,
     excluded_artifact_types: List[orm.BuildArtifactType] = None,
@@ -129,10 +156,10 @@ def list_build_artifacts(
             func.not_(orm.BuildArtifact.artifact_type.in_(excluded_artifact_types))
         )
 
-    return db.query(orm.BuildArtifact).filter(*filters).limit(limit).all()
+    return db.query(orm.BuildArtifact).filter(*filters)
 
 
-def get_build_artifact(db, build_id, key):
+def get_build_artifact(db, build_id: int, key: str):
     return (
         db.query(orm.BuildArtifact)
         .filter(orm.BuildArtifact.build_id == build_id, orm.BuildArtifact.key == key)
@@ -140,18 +167,25 @@ def get_build_artifact(db, build_id, key):
     )
 
 
-def list_conda_channels(db, limit=25):
-    return db.query(orm.CondaChannel).limit(limit).all()
+def list_conda_channels(db):
+    filters = []
+    return db.query(orm.CondaChannel).filter(*filters)
 
 
-def get_conda_channel(db, channel_name):
+def get_conda_channel(db, channel_name: str):
     return (
         db.query(orm.CondaChannel).filter(orm.CondaChannel.name == channel_name).first()
     )
 
 
-def list_conda_packages(db, limit=25):
-    return db.query(orm.CondaPackage).limit(limit).all()
+def list_conda_packages(db, search: str = None, build: str = None):
+    filters = []
+    if search:
+        filters.append(orm.CondaPackage.name.contains(search, autoescape=True))
+    if build:
+        filters.append(orm.CondaPackage.build.contains(build, autoescape=True))
+
+    return db.query(orm.CondaPackage).join(orm.CondaChannel).filter(*filters)
 
 
 def get_metrics(db):
