@@ -49,7 +49,7 @@ class CondaStore(LoggingConfigurable):
 
     database_url = Unicode(
         "sqlite:///conda-store.sqlite",
-        help="url for the database. e.g. 'sqlite:///conda-store.sqlite'",
+        help="url for the database. e.g. 'sqlite:///conda-store.sqlite' tables will be automatically created if they do not exist",
         config=True,
     )
 
@@ -258,26 +258,31 @@ class CondaStore(LoggingConfigurable):
             if not force_build:
                 return
 
-        build = self.create_build(namespace.id, specification.sha256)
-
         # Create Environment if specification of given namespace/name
         # does not exist yet
         environment = api.get_environment(
             self.db, namespace_id=namespace.id, name=specification.name
         )
-        if environment is None:
-            self.db.add(
-                orm.Environment(
-                    name=specification.name,
-                    namespace_id=namespace.id,
-                    build_id=build.id,
-                )
+        environment_was_empty = environment is None
+        if environment_was_empty:
+            environment = orm.Environment(
+                name=specification.name,
+                namespace_id=namespace.id,
             )
+            self.db.add(environment)
             self.db.commit()
 
-    def create_build(self, namespace_id: int, specification_sha256: str):
+        build = self.create_build(environment.id, specification.sha256)
+
+        if environment_was_empty:
+            environment.current_build = build
+            self.db.commit()
+
+    def create_build(self, environment_id: int, specification_sha256: str):
         specification = api.get_specification(self.db, specification_sha256)
-        build = orm.Build(namespace_id=namespace_id, specification_id=specification.id)
+        build = orm.Build(
+            environment_id=environment_id, specification_id=specification.id
+        )
         self.db.add(build)
         self.db.commit()
 
@@ -325,7 +330,7 @@ class CondaStore(LoggingConfigurable):
                 "cannot update environment to build id since specification does not match environment name"
             )
 
-        environment.build_id = build.id
+        environment.current_build_id = build.id
         self.db.commit()
 
         self.celery_app
@@ -334,10 +339,52 @@ class CondaStore(LoggingConfigurable):
 
         tasks.task_update_environment_build.si(environment.id).apply_async()
 
+    def delete_namespace(self, namespace):
+        namespace = api.get_namespace(self.db, name=namespace)
+        if namespace is None:
+            raise utils.CondaStoreError(f"namespace={namespace} does not exist")
+
+        utcnow = datetime.datetime.utcnow()
+        namespace.deleted_on = utcnow
+        for environment in namespace.environments:
+            environment.deleted_on = utcnow
+            for build in environment.builds:
+                build.deleted_on = utcnow
+        self.db.commit()
+
+        self.celery_app
+
+        # must import tasks after a celery app has been initialized
+        from conda_store_server.worker import tasks
+
+        tasks.task_delete_namespace.si(namespace.id).apply_async()
+
+    def delete_environment(self, namespace, name):
+        environment = api.get_environment(self.db, namespace=namespace, name=name)
+        if environment is None:
+            raise utils.CondaStoreError(
+                f"environment namespace={namespace} name={name} does not exist"
+            )
+
+        utcnow = datetime.datetime.utcnow()
+        environment.deleted_on = utcnow
+        for build in environment.builds:
+            build.deleted_on = utcnow
+        self.db.commit()
+
+        self.celery_app
+
+        # must import tasks after a celery app has been initialized
+        from conda_store_server.worker import tasks
+
+        tasks.task_delete_environment.si(environment.id).apply_async()
+
     def delete_build(self, build_id):
         build = api.get_build(self.db, build_id)
         if build.status not in [orm.BuildStatus.FAILED, orm.BuildStatus.COMPLETED]:
-            raise ValueError("cannot delete build since not finished building")
+            raise utils.CondaStoreError(
+                "cannot delete build since not finished building"
+            )
 
         build.deleted_on = datetime.datetime.utcnow()
         self.db.commit()
