@@ -3,8 +3,10 @@ import os
 import shutil
 
 import minio
+from minio.credentials.providers import Provider
+
 from traitlets.config import LoggingConfigurable
-from traitlets import Unicode, Bool
+from traitlets import Unicode, Bool, Type, Dict, List
 
 from conda_store_server import orm, api
 
@@ -47,22 +49,24 @@ class Storage(LoggingConfigurable):
 
 class S3Storage(Storage):
     internal_endpoint = Unicode(
-        help="internal endpoint to reach s3 bucket e.g. 'minio:9000'",
+        help="internal endpoint to reach s3 bucket e.g. 'minio:9000' this is the url that conda-store use for get/set s3 blobs",
         config=True,
     )
 
     external_endpoint = Unicode(
-        help="internal endpoint to reach s3 bucket e.g. 'localhost:9000'",
+        help="external endpoint to reach s3 bucket e.g. 'localhost:9000' this is the url that users use for fetching s3 blobs",
         config=True,
     )
 
     access_key = Unicode(
         help="access key for S3 bucket",
+        allow_none=True,
         config=True,
     )
 
     secret_key = Unicode(
         help="secret key for S3 bucket",
+        allow_none=True,
         config=True,
     )
 
@@ -84,6 +88,33 @@ class S3Storage(Storage):
         config=True,
     )
 
+    credentials = Type(
+        klass=Provider,
+        default_value=None,
+        help="provider to use to get credentials for s3 access. see examples https://github.com/minio/minio-py/tree/master/examples and documentation https://github.com/minio/minio-py/blob/master/docs/API.md#1-constructor",
+        allow_none=True,
+        config=True,
+    )
+
+    credentials_args = List(
+        [],
+        help="arguments to pass to Provider",
+        config=True,
+    )
+
+    credentials_kwargs = Dict(
+        {},
+        help="keyword arguments to pass to Provider",
+        config=True,
+    )
+
+    @property
+    def _credentials(self):
+        if self.credentials is None:
+            return None
+
+        return self.credentials(*self.credentials_args, **self.credentials_kwargs)
+
     @property
     def internal_client(self):
         if hasattr(self, "_internal_client"):
@@ -98,6 +129,7 @@ class S3Storage(Storage):
             self.secret_key,
             region=self.region,
             secure=self.secure,
+            credentials=self._credentials,
         )
         self._check_bucket_exists()
         return self._internal_client
@@ -116,6 +148,7 @@ class S3Storage(Storage):
             self.secret_key,
             region=self.region,
             secure=self.secure,
+            credentials=self._credentials,
         )
         return self._external_client
 
